/*
 * array.c - Associative arrays
 *
 * Written 2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#include <inttypes.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include "data.h"
#include "util.h"
#include "array.h"


struct array {
    struct value index;
    struct value value;
    struct array *next;
};


struct value copy_if_array(struct value value)
{
    struct value new;
    const struct array *walk;
    struct array **link;

    if (value.type != &array_type) return value;
    new = value;
    link = (struct array **) &new.value;
    for (walk = (struct array *) value.value; walk; walk = walk->next) {
	*link = alloc_t(struct array);
	(*link)->index = walk->index;
	(*link)->value = copy_if_array(walk->value);
	link = &(*link)->next;
    }
    *link = NULL;
    return new;   
}


struct value index_array(struct value *value,struct value index)
{
    struct value lvalue;
    struct array *walk,*new;

    assert(value->type == &array_type);
    if (!index.type) {
	fprintf(stderr,"undefined value in index\n");
	exit(1);
    }
    if (index.type != &string_type &&
      index.type->kind != dwx_uint && index.type->kind != dwx_sint) {
	fprintf(stderr,"invalid index type\n");
	exit(1);
    }
    for (walk = (struct array *) value->value; walk; walk = walk->next) {
	if (index.type != walk->index.type &&
	  (index.type->kind != dwx_uint || walk->index.type->kind != dwx_sint)
	  &&
	  (index.type->kind != dwx_sint || walk->index.type->kind != dwx_uint))
	    continue;
	  /* @@@ not entirely correct: should consider sign bit @@@ */
	  /* (really ? C doesn't do this either ...) */
	if (index.type == &string_type) {
	    if (!strcmp(index.value,walk->index.value)) break;
	}
	else if (*(uint32_t *) index.value == *(uint32_t *) walk->index.value)
		break;
    }
    lvalue.type = &lvalue_type;
    if (walk) {
	lvalue.value = &walk->value;
	return lvalue;
    }
    new = alloc_t(struct array);
    new->index = index;
    new->value.type = NULL;
    new->next = walk = (struct array *) value->value;
    value->value = new;
    lvalue.value = &new->value;
    return lvalue;
}
