/*
 * data.h - Values
 *
 * Written 2002,2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#ifndef DATA_H
#define DATA_H

#include <inttypes.h>

#include "dwarfx.h"


struct value {
    const struct dwx_type *type;	/* type of value; NULL if none */
    const void *value;			/* pointer to value */
};


#include "op.h"	/* must follow definition of struct value ! */


extern const struct dwx_type *int8_type;
extern const struct dwx_type *int16_type;
extern const struct dwx_type *int32_type;
extern const struct dwx_type *int64_type;
extern const struct dwx_type *uint8_type;
extern const struct dwx_type *uint16_type;
extern const struct dwx_type *uint32_type;
extern const struct dwx_type *uint64_type;

extern struct dwx_type type_type;
extern struct dwx_type string_type;
extern struct dwx_type lvalue_type;
extern struct dwx_type array_type;


struct value int_value(uint32_t num);
uint32_t value_int(struct value value);
uint32_t int_eval(const struct op *op);

struct value string_value(const char *str);
const char *value_string(struct value value);
const char *string_eval(const struct op *op);

struct value type_value(const struct dwx_type *type);
const struct dwx_type *value_type(struct value value);
const struct dwx_type *type_eval(const struct op *op);

struct value array_value(void);

struct value evaluate_lvalue(const struct op *op);
struct value dereference_lvalue(struct value value);
struct value evaluate(const struct op *op);

void init_types(void);

#endif /* DATA_H */
