%{
/*
 * script.l - Dump script language
 *
 * Written 2002,2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#include <inttypes.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "data.h"
#include "util.h"

#include "y.tab.h"


int lineno = 1;

%}

%%

unsigned			return TOK_UNSIGNED;
signed				return TOK_SIGNED;
void				return TOK_VOID;
char				return TOK_CHAR;
short				return TOK_SHORT;
int				return TOK_INT;
long				return TOK_LONG;
int8_t				return TOK_INT8_T;
uint8_t				return TOK_UINT8_T;
int16_t				return TOK_INT16_T;
uint16_t			return TOK_UINT16_T;
int32_t				return TOK_INT32_T;
uint32_t			return TOK_UINT32_T;
int64_t				return TOK_INT64_T;
uint64_t			return TOK_UINT64_T;
struct				return TOK_STRUCT;
union				return TOK_UNION;
enum				return TOK_ENUM;

print				return TOK_PRINT;
printf				return TOK_PRINTF;
without				return TOK_WITHOUT;
read				return TOK_READ;
if				return TOK_IF;
else				return TOK_ELSE;
while				return TOK_WHILE;
break				return TOK_BREAK;
exit				return TOK_EXIT;
die				return TOK_DIE;

"||"				return TOK_OR;
&&				return TOK_AND;
"<<"				return TOK_SHL;
">>"				return TOK_SHR;
==				return TOK_EQ;
!=				return TOK_NE;
"<="				return TOK_LE;
">="				return TOK_GE;

sizeof				return TOK_SIZEOF;

[a-zA-Z_][a-zA-Z_0-9]*		{ int value;
				  switch (dwx_lookup_enum(yytext,&value)) {
				 	case 0:
					    break;
					case 1:
					    yylval.num = value;
					    return ENUM;
					case 2:
					    yyerror("conflicting values");
					default:
					abort();
				  }
				  yylval.str = stralloc(yytext);
				  return WORD; }
0[Xx][0-9A-Za-z]+|0[0-7]+|[0-9]+ { char *end;
				  yylval.num = strtoul(yytext,&end,0);
				  if (*end) yyerror("invalid digit in number");
				  return NUMBER; }
\"[^\"\n\t]*\"			{ *strrchr(yytext,'"') = 0;
				  yylval.str = stralloc(yytext+1);
				  return STRING; }
\$[a-zA-Z_][a-zA-Z_0-9]*	{ yylval.var = var_lookup(yytext+1);
				  return VARIABLE; }

[\t ]*
\n				lineno++;

^#\ [0-9]+\ \"[^"]*\"(\ [0-9]+)*\n lineno = strtol(yytext+2,NULL,0);

.				return *yytext;

%%


void yyerror(const char *s)
{
    fprintf(stderr,"%d: %s near \"%s\"\n",lineno,s,yytext);
    exit(1);
}
