/*
 * stmt.h - Statement construction and execution
 *
 * Written 2002,2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#ifndef STMT_H
#define STMT_H

#include "dwarfx.h"

#include "op.h"
#include "var.h"


struct without {
    const struct op *expr;
    struct without *next;
};

struct arg_list {
    const struct op *expr;
    struct value value; /* need to cache values */
    struct arg_list *next;
};

struct stmt {
    void (*fn)(const struct stmt *self);
    union {
	struct {
	    const struct op *expr;
	} expr;
	struct {
	    const struct arg_list *args;
	    struct without *without;
	} print;
	struct {
	    const struct op *fmt;
	    struct arg_list *args;
	} printf;
	struct {
	    const struct op *expr;
	    struct stmt *true;
	    struct stmt *false;
	} if_stmt;
	struct {
	    const struct stmt *loop;
		/* before fixup: points to beginning of loop */
		/* after fixup: points to statement after loop */
	} break_stmt;
	struct {
	    const struct op *expr;
	    struct stmt *body;
	} while_stmt;
	struct {
	    const struct op *expr; /* NULL if none */
	} exit;
	struct {
	    const struct op *expr; /* NULL if none */
	    int line;
	} die;
	const char *str;
    } u;
    struct stmt *next;
};


void fix_statements(struct stmt **stmt);

void stmt_expression(const struct stmt *self);
void stmt_print(const struct stmt *self);
void stmt_printf(const struct stmt *self);
void stmt_read(const struct stmt *self);
void stmt_if(const struct stmt *self);
void stmt_break(const struct stmt *self);
void stmt_exit(const struct stmt *self);
void stmt_die(const struct stmt *self);
void stmt_while(const struct stmt *self);

void execute(const struct stmt *pgm);

#endif /* STMT_H */
